import { Element, js2xml, xml2js } from "xml-js";

import { DigitalCalibrationCertificateType, DCC } from "./DCC";
import { DSI } from "./DSI";

export class DCCDocument {
  _declaration: unknown;
  digitalCalibrationCertificate: DigitalCalibrationCertificateType;

  constructor(el?: Partial<DCCDocument>) {
    this._declaration = el?._declaration ?? { _attr: { version: "1.0", encoding: "UTF-8" } };
    this.digitalCalibrationCertificate = new DigitalCalibrationCertificateType(el?.digitalCalibrationCertificate);
  }

  static createEmpty() {
    return new DCCDocument({
      digitalCalibrationCertificate: new DigitalCalibrationCertificateType({
        _attr: {
          "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
          "xsi:schemaLocation": `${DCC.namespaceUrl} ${DCC.schemaLocation}`,
          "xmlns:dcc": DCC.namespaceUrl,
          "xmlns:si": DSI.namespaceUrl,
          schemaVersion: DCC.schemaVersion,
        },
      }),
    });
  }

  static fromXml(xml) {
    const parsed = <Partial<DCCDocument>>xml2js(xml, {
      compact: true,
      ignoreComment: true,
      trim: true,
      attributesKey: "_attr",
      elementNameFn: function (val) {
        return val.replace("dcc:", "").replace("si:", "");
      },
      textFn: function (val) {
        return val
          .split("\n")
          .map((x) => x.trim())
          .join("\n");
      },
    });

    return new DCCDocument(parsed);
  }

  toXML(): string {
    /* Properties order in objects is not guaranteed in JavaScript.
     *  This is a workaround to fix this.
     *  The keys are set in the right order by creating a new instance of DCCDocument */
    const fixedOrderDoc = new DCCDocument(this);
    return js2xml(<Element>fixedOrderDoc, {
      compact: true,
      ignoreComment: true,
      spaces: 2,
      attributesKey: "_attr",
      elementNameFn: function (val, currentElementObj) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const currentElement = <any>currentElementObj;
        return `${currentElement?.namespace ? `${currentElement.namespace}:` : ""}${val}`;
      },
    });
  }
}
